require 'rubygems'
require 'gosu'
require 'singleton'

require './lib/manager/graphics_manager.rb'
require './lib/manager/sound_manager.rb'
require './lib/map.rb'
require './lib/map_tile.rb'
require './lib/dirt_tile.rb'
require './lib/wall_tile.rb'

require './player.rb'
require './bullet.rb'
require './zombie.rb'
require './blood_splat.rb'

class Camera

  def initialize(window, x, y)
    @window = window

    @x = x
    @y = y

    @speed = 10
  end

  def look_at(thing)
    # puts "Camera offset: #{@x-thing.x}, #{@y-thing.y}"

    @x = thing.x
    @y = thing.y

    # if @x-thing.x < @speed*20
    #   @x += @speed
    # elsif  @x-thing.x > @speed*20
    #   @x -= @speed
    # end
    #
    # if @y-thing.y < @speed*20
    #   @y += @speed
    # elsif @y-thing.y > @speed*20
    #   @y -= @speed
    # end

  end

  def x
    @x - @window.width / 2
  end

  def y
    @y - @window.height / 2
  end

end

class ZombiesGame < Gosu::Window

  def initialize(width=800, height=600, fullscreen=false)
    super(1024, 768, fullscreen)
    self.caption = "Zombies!"
    $game = self

    GraphicsManager.instance

    @map = Map.new(self)
    @camera = Camera.new(self, 20, 20)
    @player = Player.new(self, @map.width/2, @map.height/2)
    @max_zombies = 20
    @paused = false

    @bullets = []
    @zombies = []
    @blood_splats = []

    @font = Gosu::Font.new(self, Gosu::default_font_name, 18)
    @huge_font = Gosu::Font.new(self, Gosu::default_font_name, 58)
  end

  def needs_cursor?
    true
  end

  def button_down(id)
    self.close if id == Gosu::KbEscape

    @paused = !@paused if id == Gosu::KbP
  end

  def button_up(id)
    # ...
  end

  def update
    return if @paused

    unless @player.killed?
      @player.update(mouse_x+@camera.x, mouse_y+@camera.y)

      # TODO limit speed during vertical movement!!!
      @player.move_up    if self.button_down? Gosu::KbW
      @player.move_down  if self.button_down? Gosu::KbS
      @player.move_left  if self.button_down? Gosu::KbA
      @player.move_right if self.button_down? Gosu::KbD

      add_bullet(@player.fire) if self.button_down? Gosu::MsLeft
    end

    if @zombies.length < max_zombies
      @zombies += Zombie.spawn_around(@player, max_zombies - @zombies.length, self)
    end

    # Move zombies ------------------------------------------------------------
    @zombies.each do |z|
      z.update(@player)
    end

    # Check bullet/zombie collisions ------------------------------------------
    dead_bullets = []
    dead_zombies = []
    @bullets.each do |b|
      @zombies.each do |z|
        if b.collide?(z)
          dead_bullets << b
          z.hit(b.power)
          if z.killed?
            dead_zombies << z
            z.die
          end
          break
        end
      end
    end

    dead_zombies.each do |zombie|
      @blood_splats << BloodSplat.new(self, zombie.x, zombie.y, zombie.angle)
    end

    @bullets -= dead_bullets
    @zombies -= dead_zombies

    @player.kill_count += dead_zombies.length

    # Check zombie/player collisions ------------------------------------------
    @zombies.each do |z|
      if z.collide?(@player)
        @player.hit(z.power)
      end
    end

    @bullets = @bullets.reject(&:end_of_life)
    @bullets.each(&:update)

    @camera.look_at(@player)
  end

  def draw
    translate -@camera.x, -@camera.y do
      @map.draw(@camera)
      @blood_splats.each(&:draw)
      @player.draw
      @zombies.each(&:draw)
      @bullets.each(&:draw)
    end

    draw_hud
  end

  def add_bullet(bullet)
    @bullets << bullet if bullet
  end

  def max_zombies
    @max_zombies + (@player.kill_count * 0.2).to_i
  end

  def draw_hud
    @font.draw_rel("Zombies killed: #{@player.kill_count}", self.width/2, self.height-60, 0, 1.0, 0.0)
    @font.draw_rel("Health: #{@player.health}", self.width/2, self.height-20, 0, 1.0, 0.0)
    if @player.killed?
      @huge_font.draw_rel("You're dead!", self.width/2, self.height/2, 0, 1.0, 0.0)
    end
  end

end

ZombiesGame.new.show
